﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum AttackType
{
    Laser,
    Bomb,
    Bullet
}

public class Boss : MonoBehaviour {

    private Transform tr;
    
    public Vector2 initialPosition;
    public Vector2 finalPosition;
    public float entranceSpeed;

    public AttackDescription[] attacks;
    private float attackTotProb;

    public float initialWaitingTime;
    private float waitingTime;
    private float timeCounter;

    private bool endedEntrance;
    private bool dead;

    public int lifes;
    public float scalingSpeed;
    
    void Start()
    {
        GameManager.Instance.AddGameObjectToTheScreen(gameObject);

        tr = GetComponent<Transform>();
        tr.position = initialPosition;

        attackTotProb = 0f;

        foreach (AttackDescription ad in attacks)
            attackTotProb += ad.probability;

        waitingTime = initialWaitingTime;
        timeCounter = 0f;

        endedEntrance = false;
        dead = false;

        StartCoroutine(Entrance());
    }

    private IEnumerator Entrance()
    {
        do
        {
            tr.position = Vector3.MoveTowards(tr.position, finalPosition, entranceSpeed);
            yield return new WaitForEndOfFrame();
        } while (Vector3.Distance(tr.position, finalPosition) > 0);

        endedEntrance = true;
    }

    void Update()
    {
        if (!endedEntrance)
            return;

        timeCounter += Time.deltaTime;

        if (timeCounter >= waitingTime && !dead)
        {
            Attack();
            timeCounter = 0f;
        }
    }

    void OnDestroy()
    {
        GameManager.Instance.RemoveGameObjectFromTheScreen(gameObject);
    }

    private void Attack()
    {
        float rand = Random.Range(0, attackTotProb);

        foreach (AttackDescription ad in attacks)
            if (ad.probability > rand)
            {
                waitingTime = ad.waitingTimeAfterAttack;
                SwitchAttack(ad);
                break;
            }
            else
                rand -= ad.probability;
    }

    private void SwitchAttack(AttackDescription ad)
    {
        switch(ad.type)
        {
            case AttackType.Laser: Laser(ad.go); break;
            case AttackType.Bomb: Bomb(ad.go); break;
            case AttackType.Bullet: Bullet(ad.go); break;
        }
    }

    private void Laser(GameObject go)
    {
        float y = GameManager.Instance.GetPossibleY();
        Instantiate(go, new Vector2(tr.position.x, y), Quaternion.identity);
    }

    private void Bomb(GameObject go)
    {
        float y = GameManager.Instance.GetPossibleY();
        Instantiate(go, new Vector2(tr.position.x, y), Quaternion.identity);
    }

    private void Bullet(GameObject go)
    {
        float y = GameManager.Instance.GetPossibleY();
        Instantiate(go, new Vector2(tr.position.x, y), Quaternion.identity);
    }

    public void Damage()
    {
        lifes--;

        POLIMIGameCollective.SfxManager.Instance.Play("sfx_bossDeath");

        if (lifes <= 0)
            StartCoroutine(Die());
    }

    private IEnumerator Die()
    {
        dead = true;
        GameObject.FindGameObjectWithTag("Wave").GetComponent<Player>().GoForward();

        ;

        Vector3 scale = tr.localScale;
        do
        {
            scale.x -= scalingSpeed;
            tr.localScale = scale;
            yield return new WaitForEndOfFrame();
        } while (scale.x > 0f);

        SpawnManager.Instance.RespawnPlayer(GameObject.FindGameObjectWithTag("Player").GetComponent<Boat>(), true);
        gameObject.SetActive(false);
    }

    [System.Serializable]
    public struct AttackDescription
    {
        public AttackType type;
        public GameObject go;
        public float waitingTimeAfterAttack;
        public float probability;

        //TODO decidere se vogliamo una vita del giocatore o si sciotta in un colpo solo.
    }
}
