﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

public class GameManager : MonoBehaviour {
    private List<GameObject> gos = new List<GameObject>();
    
    public Vector2 worldBound;

    private bool playerAlive;
    public bool PlayerAlive
    {
        get { return playerAlive; }
        set { playerAlive = value; }
    }

    private static GameManager instance;
    public static GameManager Instance
    {
        get { return instance; }
    }

    private float multiplier; // for the speed of the enemy [value from 1 to inf]
    public float Multiplier
    {
        get { return multiplier; }
    }

    private float divider; // for the times of the spawns [value from 0 to 1]
    public float Divider
    {
        get { return divider; }
    }

    public float initialWaitingTime;
    private float waitingTime;
    private float timeCounter;

    public float minDeltaTimeEnemySpawn;
    public float maxDeltaTimeEnemySpawn;

    public float minDeltaTimePatternSpawn;
    public float maxDeltaTimePatternSpawn;

    private int numberOfEnemies;

    public Pattern[] patterns;

    [Range(0f, 1f)]
    public float difficultyIncreaser;

    private float enemyWavesTotProb;

    public float spawnX;

    public int numberOfRaws;
    public float minSpawnY, maxSpawnY;
    private float rawHeight;

    public GameObject boss;

    private bool callTheBoss;
    private bool bossBattle;

    private bool gameplay;
    public bool Gameplay
    {
        get { return gameplay; }
    }

    public class TheStart : UnityEvent { }
    public TheStart theStart = new TheStart();
    public TheStart theStartWorld = new TheStart();

    // gestisce il singleton
    void Awake () {
        if (instance == null)
            instance = this;
        else
            Destroy(gameObject);
	}

    // segnala un elemento aggiunto alla scena (e lo aggiunge alla rispettiva lista)
    public void AddGameObjectToTheScreen(GameObject go)
    {
        gos.Add(go);
    }

    // segnala un elemento tolto dalla scena (e lo toglie dalla rispettiva lista)
    public void RemoveGameObjectFromTheScreen(GameObject go)
    {
        gos.Remove(go);
    }

    // setta variabili iniziali tra cui i parametri per il calcolo delle probabilità
    void Start()
    {
        PlayerAlive = true;
        gameplay = false;

        waitingTime = initialWaitingTime;
        timeCounter = 0f;

        enemyWavesTotProb = 0f;

        foreach (Pattern ew in patterns)
            enemyWavesTotProb += ew.probability;

        rawHeight = (maxSpawnY - minSpawnY) / numberOfRaws;

        multiplier = 1f;
        divider = 1f;

        callTheBoss = false;
        bossBattle = false;
    }
	
    // se non siamo in una bossBattle, il giocatore è vivo e siamo in fase di gameplay genera i pattern
	void Update () {
        if (bossBattle || !PlayerAlive || !gameplay)
            return;

        timeCounter += Time.deltaTime;

        if (timeCounter >= waitingTime)
        {
            if (callTheBoss)
                SpawnBoss();
            else
            {
                CreateNewWaveOfEnemies();
                timeCounter = 0f;
            }
        }
    }

    // creazione del pattern tramite la generazione di una colonna alla volta
    private void CreateNewWaveOfEnemies ()
    {
        float rand = Random.Range(0, enemyWavesTotProb);
        numberOfEnemies = 0;

        foreach (Pattern pt in patterns)
            if (pt.probability > rand)
            {
                float waitingTimeAfter = Random.Range(minDeltaTimePatternSpawn, maxDeltaTimePatternSpawn);
                waitingTime = (waitingTimeAfter + pt.columns.Length * maxDeltaTimeEnemySpawn)  / Divider;
                StartCoroutine(CreateEnemies(pt.columns));
                break;
            }
            else
                rand -= pt.probability;
    }

    // genera tutti i nemici di una colonna
    private IEnumerator CreateEnemies (Column[] columns)
    {
        bool[] raws = new bool[numberOfRaws];

        foreach (Column column in columns)
        {
            for (int i = 0; i < numberOfRaws; raws[i] = true, i++) ;

            foreach (Column.ObjectSpawn os in column.objects)
                if (os.y >= 0 && os.y < numberOfRaws && raws[os.y]) {
                    Instantiate(os.obj, new Vector2(spawnX, minSpawnY + rawHeight * os.y), Quaternion.identity);
                    raws[os.y] = false;
                }


            float deltaTimeEnemySpawn = Random.Range(minDeltaTimeEnemySpawn, maxDeltaTimeEnemySpawn);
            yield return new WaitForSeconds(deltaTimeEnemySpawn  / Divider);
        }
    }

    // segnala l'inizio della fase di gameplay
    public void StartGameplay()
    {
        gameplay = true;
        theStartWorld.Invoke();
    }

    // segnala ed aumenta la difficoltà
    public void IncreaseDifficulty()
    {
        multiplier += difficultyIncreaser;
        divider += difficultyIncreaser;
    }

    // setta la chiamata di un boss se non è già stato chiamato
    public void CallTheBoss()
    {
        if (!callTheBoss && !bossBattle)
            callTheBoss = true;
    }

    // creaa il boss
    private void SpawnBoss()
    {
        callTheBoss = false;
        bossBattle = true;

        Instantiate(boss);
    }

    // restituisce una possibile posizione sull'asse Y
    public float GetPossibleY()
    {
        int i =  Random.Range(0, 3);

        ;

        return minSpawnY + rawHeight * i;
    }

    // resetta il livello togliendo tutti i nemici e le armi; in più resetta la difficoltà
    public void CleanScreen()
    {
        divider = 1;
        multiplier = 1;

        foreach(GameObject go in gos)
            Destroy(go);

        gos.Clear();
    }
}
