﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Player : MonoBehaviour {

    private Transform tr;
    private Rigidbody2D rb;

    [Header("Jump")]
    [Range(0f, 100f)]
    public float m_jumpForce = 8;

    [Header("Gravity")]
    [Range(0f, 100f)]
    public float m_gravityForce = 3f;

    private bool jump;

    public float minPosY = -5.5f;
    public float maxPosY = -3f;
    int i = 0;
    private static float GravityVelocity = 3f;

    [Range(0f, 100f)]
    public float m_speedDown = 3f;

    private bool endLevel;
    public bool isEndLevel()
    {
        return endLevel;
    }

    [Header("Start/Ending Game")]
    public float speedEndingLevel;
    
    public CanvasGroup[] names;
    public float durationEachName;
    public float hidingNamesSpeed;

    private bool canStartGameplay;

    void Start () {
        tr = GetComponent<Transform>() as Transform;
        rb = GetComponent<Rigidbody2D>() as Rigidbody2D;
        
        endLevel = false;

        foreach (CanvasGroup n in names)
            n.alpha = 0f;

        GameManager.Instance.theStart.AddListener(TheStart);

        canStartGameplay = false;
    }

    public void TheStart()
    {
        StartCoroutine(Opening());
        //canStartGameplay = true;
    }

    // gestisce i titoli di testa facendoli comparire e poi chiama una coroutine che li fa sparire
    private IEnumerator Opening()
    {
        float openingEnd = Time.time + durationEachName * (names.Length + 1); // parametri per finire senza scattare
        
        float openingCounter = - durationEachName; // per partire subito
        int i = -1;

        do
        {

            if(Time.time - openingCounter >= durationEachName)
            {
                if (i >= 0 && i < names.Length)
                    StartCoroutine(HideName(names[i]));

                i++;
                if (i >= 0 && i < names.Length)
                    names[i].alpha = 1f;

                openingCounter = Time.time;
            }

            yield return new WaitForEndOfFrame();
        } while (Time.time < openingEnd);

        canStartGameplay = true;
    }

    // fa sparire ogni nome dei titoli di testa agendo sull'alpha del canvas group
    private IEnumerator HideName(CanvasGroup cg)
    {
        do
        {
            cg.alpha = cg.alpha - hidingNamesSpeed;
            yield return new WaitForEndOfFrame();
        } while (cg.alpha > 0);
    }
	
    // se non si è in fase di noGameplay (titoli testa) fa qualcosa:
        // se siamo a fine livello sposta in avanti
        // e prende l'input da barra spaziatrice
	void Update () {
        if (!GameManager.Instance.Gameplay)
        {
            tr.position = Vector3.right * tr.position.x + Vector3.up * (minPosY + (maxPosY - minPosY) * (Mathf.Sin(Time.time) + 1) / 2);

            if(canStartGameplay && tr.position.y <= minPosY + 0.05f) // posizione minima + rumore
                GameManager.Instance.StartGameplay();

            return;
        }

        if(endLevel)
        {
            rb.velocity = new Vector3(speedEndingLevel, rb.velocity.y, 0);// tr.position = tr.position + Vector3.right * speedEndingLevel * Time.deltaTime;
        }

        jump = Input.GetKey(KeyCode.Space);
    }

    // gestisce i movimenti dell'onda se non siamo in zona noGameplay
    private void FixedUpdate()
    {
        if (!GameManager.Instance.Gameplay)
            return;

        if (((tr.position.y >= maxPosY) && (rb.velocity.y > 0)) || ((tr.position.y <= minPosY) && (rb.velocity.y < 0)))
        {
            rb.velocity = new Vector2(0f, 0f);
        }
        else if ((jump) && (tr.position.y < maxPosY))
        {
            rb.AddForce(new Vector2(0, m_jumpForce), ForceMode2D.Force);
        }
        else if ((!jump) && (tr.position.y > minPosY))
        {
            rb.AddForce(new Vector2(0, -m_gravityForce), ForceMode2D.Force);
        }
    }

    // aziona l'animazione di fine livello
    public void GoForward()
    {
        endLevel = true;
    }
}
