﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SpawnManager : MonoBehaviour {
    private static SpawnManager instance;
    public static SpawnManager Instance
    {
        get { return instance; }
    }

    private bool waitingForInput;

    private Boat player;

    public CanvasGroup endGamePanel;
    public Animator textAnimator;
    public float showHideSpeed;

    // singleton
    void Awake()
    {
        if (Instance == null)
            instance = this;
        else
            Destroy(gameObject);
    }

    // panello di gameover nascosto e comandi disattivati
    void Start()
    {
        waitingForInput = false;

        endGamePanel.alpha = 0;
    }

    // parte coroutine per respawnare la barca (quando si preme spazio)
    void Update()
    {
        if(Input.GetKeyDown(KeyCode.Space) && waitingForInput)
            StartCoroutine(HideEndGameAndStart());
    }

    // segnala la fine di una partita
    public void RespawnPlayer(Boat player, bool win)
    {
        this.player = player;

        if (!win)
            StartCoroutine(ShowEndGame());
        else
            waitingForInput = true;
    }

    // compare la schermata di gameover
    private IEnumerator ShowEndGame()
    {
        POLIMIGameCollective.SfxManager.Instance.Play("jng_gameOver");
        yield return new WaitForSeconds(0.5f);

        do
        {
            endGamePanel.alpha += showHideSpeed;
            yield return new WaitForEndOfFrame();
        } while (endGamePanel.alpha < 1f);

        textAnimator.SetTrigger("Text");
        waitingForInput = true;
    }

    // nasconde la schermata di gameover e respawna il giocatore
    private IEnumerator HideEndGameAndStart()
    {
        waitingForInput = false;

        GameManager.Instance.CleanScreen();
        PointsManager.Instance.ResetPoints();

        do
        {
            endGamePanel.alpha -= showHideSpeed;
            yield return new WaitForEndOfFrame();
        } while (endGamePanel.alpha > 0f);

        textAnimator.SetTrigger("Reset");

        player.Reset();
    }
}
