﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider2D))]
public class Weapon : MonoBehaviour {
    protected Transform tr;

    protected bool inversingSpeed;
    protected bool inverseSpeed;
    public float rotationSpeed;

    public GameObject explosion;
    public float offset;

    // segnala l'oggetto in scena
    protected void Start()
    {
        GameManager.Instance.AddGameObjectToTheScreen(gameObject);

        inversingSpeed = false;
        inverseSpeed = false;
    }

    // toglie l'oggetto dalla scena
    void OnDestroy()
    {
        GameManager.Instance.RemoveGameObjectFromTheScreen(gameObject);
    }

    // collisioni:
        // giocatore e moto non invertito -> giocatore muore
        // onda, io che sono un oggetto scagliabile, giocatore vivo e moto non ancora invertito -> parte coroutine per invertire il moto
        // boss e moto invertito colpisce il boss, sparisce e crea esplosione
    void OnTriggerEnter2D(Collider2D other)
    {
        if (other.CompareTag("Player") && !inversingSpeed)
            other.GetComponent<Boat>().Death();
        else if (other.CompareTag("Wave") && gameObject.CompareTag("Weapon") && GameManager.Instance.PlayerAlive && !inversingSpeed)
            StartCoroutine(ChangeDirection());
        else if (other.CompareTag("Boss") && inverseSpeed)
        {
            other.GetComponent<Boss>().Damage();
            Destroy(gameObject);

            if(explosion != null)
                Instantiate(explosion, tr.position + Vector3.right * offset, Quaternion.identity);
        }
    }

    // inverte il moto di un arma
    private IEnumerator ChangeDirection()
    {
        float angle = 0f;
        inversingSpeed = true;

        POLIMIGameCollective.SfxManager.Instance.Play("voice_japaneseShout_01");

        do
        {
            angle += rotationSpeed;
            tr.Rotate(0, 0, rotationSpeed);
            yield return new WaitForEndOfFrame();
        } while (angle < 360f);

        inverseSpeed = true;

        POLIMIGameCollective.SfxManager.Instance.Play("voice_japaneseShout_02");
    }
}
